@@ -39,7 +39,8 @@ CUSTOM_SETTINGS = [
     'assume_policy',
     'attach_policy',
     'aws_region',
-    'delete_zip',
+    'delete_local_zip',
+    'delete_s3_zip',
     'exclude',
     'http_methods',
     'integration_response_codes',
@@ -106,7 +107,11 @@ class ZappaCLI(object):
     @property
     def stage_config(self):
         """A shortcut property for settings of staging."""
-        return self.zappa_settings[self.api_stage]
+        # Backwards compatible for delete_zip setting that was more explicitly named delete_local_zip
+        settings = self.zappa_settings[self.api_stage]
+        if u'delete_zip' in settings:
+            settings[u'delete_local_zip'] = settings.get(u'delete_zip')
+        return settings
 
     def handle(self, argv=None):
         """
@@ -151,10 +156,14 @@ class ZappaCLI(object):
             print("The command '{}' is not recognized. {}".format(command, help_message))
             return
 
+        # Settings-based interactions will fail
+        # before a project has been initialized.
         if command != 'init':
-            if len(command_env) < 2: # pragma: no cover
-                self.load_settings_file(vargs['settings_file'])
 
+             # Load and Validate Settings File
+            self.load_settings_file(vargs['settings_file'])
+
+            if len(command_env) < 2: # pragma: no cover
                 # If there's only one environment defined in the settings,
                 # use that as the default.
                 if len(self.zappa_settings.keys()) is 1:
@@ -165,12 +174,13 @@ class ZappaCLI(object):
             else:
                 self.api_stage = command_env[1]
 
+            if vargs['app_function'] is not None:
+                self.app_function = vargs['app_function']
+
             # Load our settings
             self.load_settings(vargs['settings_file'])
             self.callback('settings')
 
-            if vargs['app_function'] is not None:
-                self.app_function = vargs['app_function']
 
         # Hand it off
         if command == 'deploy': # pragma: no cover
@@ -297,15 +307,16 @@ class ZappaCLI(object):
                 requests.get(endpoint_url)
 
         # Finally, delete the local copy our zip package
-        if self.stage_config.get('delete_zip', True):
-            os.remove(self.zip_path)
+        if self.stage_config.get('delete_local_zip', True):
+            self.remove_local_zip()
 
         # Remove the uploaded zip from S3, because it is now registered..
-        self.zappa.remove_from_s3(self.zip_path, self.s3_bucket_name)
+        if self.stage_config.get('delete_s3_zip', True):
+            self.zappa.remove_from_s3(self.zip_path, self.s3_bucket_name)
 
         self.callback('post')
 
-        print("Deployed! {}".format(endpoint_url))
+        click.echo(click.style("Deployment complete", fg="green", bold=True) + "!: {}".format(endpoint_url))
 
 
     def update(self):
@@ -318,11 +329,15 @@ class ZappaCLI(object):
             self.execute_prebuild_script()
 
         # Temporary version check
-        updated_time = 1472581018
-        function_response = self.zappa.lambda_client.get_function(FunctionName=self.lambda_name)
-        conf = function_response['Configuration']
-        last_updated = parser.parse(conf['LastModified'])
-        last_updated_unix = time.mktime(last_updated.timetuple())
+        try:
+            updated_time = 1472581018
+            function_response = self.zappa.lambda_client.get_function(FunctionName=self.lambda_name)
+            conf = function_response['Configuration']
+            last_updated = parser.parse(conf['LastModified'])
+            last_updated_unix = time.mktime(last_updated.timetuple())
+        except Exception as e:
+            click.echo(click.style("Warning!", fg="red") + " Couldn't get function - have you deployed yet?")
+            return
 
         if last_updated_unix <= updated_time:
             click.echo(click.style("Warning!", fg="red") + " You may have upgraded Zappa since deploying this application. You will need to " + click.style("redeploy", bold=True) + " for this deployment to work properly!")
@@ -347,7 +362,8 @@ class ZappaCLI(object):
             self.s3_bucket_name, self.zip_path, self.lambda_name)
 
         # Remove the uploaded zip from S3, because it is now registered..
-        self.zappa.remove_from_s3(self.zip_path, self.s3_bucket_name)
+        if self.stage_config.get('delete_s3_zip', True):
+            self.zappa.remove_from_s3(self.zip_path, self.s3_bucket_name)
 
         # Update the configuration, in case there are changes.
         self.lambda_arn = self.zappa.update_lambda_configuration(lambda_arn=self.lambda_arn,
@@ -359,8 +375,8 @@ class ZappaCLI(object):
                                                        memory_size=self.memory_size)
 
         # Finally, delete the local copy our zip package
-        if self.stage_config.get('delete_zip', True):
-            os.remove(self.zip_path)
+        if self.stage_config.get('delete_local_zip', True):
+            self.remove_local_zip()
 
         if self.stage_config.get('domain', None):
             endpoint_url = self.stage_config.get('domain')
@@ -460,7 +476,7 @@ class ZappaCLI(object):
         if remove_logs:
             self.zappa.remove_lambda_function_logs(self.lambda_name)
 
-        print("Done!")
+        click.echo(click.style("Done", fg="green", bold=True) + "!")
 
     def schedule(self):
         """
@@ -996,7 +1012,10 @@ class ZappaCLI(object):
         """
 
         with open(settings_file) as json_file:
-            self.zappa_settings = json.load(json_file)
+            try:
+                self.zappa_settings = json.load(json_file)
+            except ValueError:
+                raise ValueError("Unable to load the zappa settings JSON. It may be malformed.")
 
     def create_package(self):
         """
@@ -1107,7 +1126,7 @@ class ZappaCLI(object):
         Remove our local zip file.
         """
 
-        if self.stage_config.get('delete_zip', True):
+        if self.stage_config.get('delete_local_zip', True):
             try:
                 os.remove(self.zip_path)
             except Exception as e: # pragma: no cover
@@ -1119,10 +1138,8 @@ class ZappaCLI(object):
         """
 
         # Remove the uploaded zip from S3, because it is now registered..
-        self.zappa.remove_from_s3(self.zip_path, self.s3_bucket_name)
-
-        # Finally, delete the local copy our zip package
-        self.remove_local_zip()
+        if self.stage_config.get('delete_s3_zip', True):
+            self.zappa.remove_from_s3(self.zip_path, self.s3_bucket_name)
 
     def print_logs(self, logs):
         """
@@ -1186,16 +1203,19 @@ def handle(): # pragma: no cover
     except SystemExit as e: # pragma: no cover
         if cli.zip_path:
             cli.remove_uploaded_zip()
+            cli.remove_local_zip()
 
         sys.exit(e.code)
 
     except KeyboardInterrupt: # pragma: no cover
         if cli.zip_path: # Remove the Zip from S3 upon failure.
             cli.remove_uploaded_zip()
+            cli.remove_local_zip()
         sys.exit(130)
     except Exception as e:
         if cli.zip_path: # Remove the Zip from S3 upon failure.
             cli.remove_uploaded_zip()
+            cli.remove_local_zip()
 
         click.echo("Oh no! An " + click.style("error occured", fg='red', bold=True) + "! :(")
         click.echo("\n==============\n")
