@@ -11,6 +11,7 @@ import inspect
 import collections
 
 import boto3
+import sys
 from werkzeug.wrappers import Response
 
 # This file may be copied into a project's root,
@@ -19,7 +20,7 @@ try:
     from zappa.cli import ZappaCLI
     from zappa.middleware import ZappaWSGIMiddleware
     from zappa.wsgi import create_wsgi_request, common_log
-except ImportError as e: # pragma: no cover
+except ImportError as e:  # pragma: no cover
     from .cli import ZappaCLI
     from .middleware import ZappaWSGIMiddleware
     from .wsgi import create_wsgi_request, common_log
@@ -31,9 +32,26 @@ logger.setLevel(logging.INFO)
 
 ERROR_CODES = [400, 401, 403, 404, 500]
 
-class LambdaException(Exception):
+
+class WSGIException(Exception):
+    """
+    This exception is used by the handler to indicate that underlying WSGI app has returned a non-2xx(3xx) code.
+    """
+
     pass
 
+
+class UncaughtWSGIException(Exception):
+    """
+    Indicates a problem that happened outside of WSGI app context (and thus wasn't handled by the WSGI app itself)
+    while processing a request from API Gateway.
+    """
+
+    def __init__(self, message, original=None):
+        super(UncaughtWSGIException, self).__init__(message)
+        self.original = original
+
+
 class LambdaHandler(object):
     """
     Singleton for avoiding duplicate setup.
@@ -48,7 +66,6 @@ class LambdaHandler(object):
 
     def __new__(cls, settings_name="zappa_settings", session=None):
         """Singleton instance to avoid repeat setup"""
-
         if LambdaHandler.__instance is None:
             LambdaHandler.__instance = object.__new__(cls, settings_name, session)
         return LambdaHandler.__instance
@@ -72,7 +89,7 @@ class LambdaHandler(object):
         try:
             os.environ["PROJECT"] = self.settings.PROJECT_NAME
             os.environ["STAGE"] = self.settings.API_STAGE
-        except Exception as e: # pragma: no cover
+        except Exception:  # pragma: no cover
             pass
 
         # Set any locally defined env vars
@@ -86,7 +103,6 @@ class LambdaHandler(object):
         sensitiZve or stage-specific configuration variables in s3 instead of
         version control.
         """
-
         if not self.session:
             boto_session = boto3.Session()
         else:
@@ -102,14 +118,14 @@ class LambdaHandler(object):
 
         try:
             content = remote_env_object['Body'].read().decode('utf-8')
-        except Exception as e: # pragma: no cover
+        except Exception as e:  # pragma: no cover
             # catch everything aws might decide to raise
             print('Exception while reading remote settings file.', e)
             return
 
         try:
             settings_dict = json.loads(content)
-        except (ValueError, TypeError): # pragma: no cover
+        except (ValueError, TypeError):  # pragma: no cover
             print('Failed to parse remote settings!')
             return
 
@@ -122,35 +138,53 @@ class LambdaHandler(object):
                 ))
             os.environ[key] = value
 
-    def import_module_and_get_function(self, whole_function):
+    @staticmethod
+    def import_module_and_get_function(whole_function):
         """
         Given a modular path to a function, import that module
         and return the function.
         """
-
         module, function = whole_function.rsplit('.', 1)
         app_module = importlib.import_module(module)
         app_function = getattr(app_module, function)
         return app_function
 
     @classmethod
-    def lambda_handler(cls, event, context): # pragma: no cover
+    def lambda_handler(cls, event, context):  # pragma: no cover
         handler = cls()
+        exception_handler = handler.settings.EXCEPTION_HANDLER
         try:
             return handler.handler(event, context)
-        except LambdaException as lex:
+        except WSGIException as wsgi_ex:
             # do nothing about LambdaExceptions since those are already handled (or should be handled by the WSGI app).
-            raise lex
+            raise wsgi_ex
+        except UncaughtWSGIException as u_wsgi_ex:
+            # hand over original error to exception handler, since the exception happened outside of WSGI app context
+            # (it wasn't propertly processed by the app itself)
+            cls._process_exception(exception_handler=exception_handler,
+                                   event=event, context=context, exception=u_wsgi_ex.original)
+            # raise unconditionally since it's an API gateway error (i.e. client expects to see a 500 and execution
+            # won't be retried).
+            raise u_wsgi_ex
         except Exception as ex:
-            exception_handler = handler.settings.EXCEPTION_HANDLER
-            if exception_handler:
-                handler_function = handler.import_module_and_get_function(exception_handler)
-                try:
-                    handler_function(ex, event, context)
-                except Exception as cex:
-                    logger.error(msg='Failed to process exception via custom handler.')
-                    print(cex)
-            raise ex
+            exception_processed = cls._process_exception(exception_handler=exception_handler,
+                                                         event=event, context=context, exception=ex)
+            if not exception_processed:
+                # Only re-raise exception if handler directed so. Allows handler to control if lambda has to retry
+                # an event execution in case of failure.
+                raise ex
+
+    @classmethod
+    def _process_exception(cls, exception_handler, event, context, exception):
+        exception_processed = False
+        if exception_handler:
+            try:
+                handler_function = cls.import_module_and_get_function(exception_handler)
+                exception_processed = handler_function(exception, event, context)
+            except Exception as cex:
+                logger.error(msg='Failed to process exception via custom handler.')
+                print(cex)
+        return exception_processed
 
     @staticmethod
     def run_function(app_function, event, context):
@@ -175,7 +209,6 @@ class LambdaHandler(object):
         """
         Call 'certify' locally.
         """
-
         import boto3
         session = boto3.Session()
 
@@ -225,7 +258,7 @@ class LambdaHandler(object):
             level = logging.getLevelName(settings.LOG_LEVEL)
             logger.setLevel(level)
 
-        # This is the result of a keep alive, recertify 
+        # This is the result of a keep alive, recertify
         # or scheduled event.
         if event.get('detail-type') == u'Scheduled Event':
 
@@ -255,9 +288,9 @@ class LambdaHandler(object):
 
             from django.core import management
 
-            try: # Support both for tests
+            try:  # Support both for tests
                 from zappa.ext.django_zappa import get_django_wsgi
-            except ImportError as e: # pragma: no cover
+            except ImportError as e:  # pragma: no cover
                 from django_zappa_app import get_django_wsgi
 
             # Get the Django WSGI app from our extension
@@ -299,9 +332,9 @@ class LambdaHandler(object):
                 trailing_slash = False
             else:
 
-                try: # Support both for tests
+                try:  # Support both for tests
                     from zappa.ext.django import get_django_wsgi
-                except ImportError as e: # pragma: no cover
+                except ImportError as e:  # pragma: no cover
                     from django_zappa_app import get_django_wsgi
 
                 # Get the Django WSGI app from our extension
@@ -330,10 +363,11 @@ class LambdaHandler(object):
                     script_name = '/' + settings.API_STAGE
 
                 # Create the environment for WSGI and handle the request
-                environ = create_wsgi_request(event,
-                                                script_name=script_name,
-                                                trailing_slash=trailing_slash
-                                            )
+                environ = create_wsgi_request(
+                    event,
+                    script_name=script_name,
+                    trailing_slash=trailing_slash
+                )
 
                 # We are always on https on Lambda, so tell our wsgi app that.
                 environ['HTTPS'] = 'on'
@@ -383,43 +417,48 @@ class LambdaHandler(object):
                 common_log(environ, response, response_time=response_time_ms)
 
                 # Finally, return the response to API Gateway.
-                if exception: # pragma: no cover
-                    raise LambdaException(exception)
+                if exception:  # pragma: no cover
+                    raise WSGIException(exception)
                 else:
                     return zappa_returndict
-        except LambdaException as e: # pragma: no cover
+        except WSGIException as e:  # pragma: no cover
             raise e
-        except Exception as e: # pragma: no cover
+        except Exception as e:  # pragma: no cover
 
             # Print statements are visible in the logs either way
             print(e)
+            exc_info = sys.exc_info()
+            message = 'An uncaught exception happened while servicing this request.'
 
             # If we didn't even build an app_module, just raise.
             if not settings.DJANGO_SETTINGS:
                 try:
                     app_module
-                except NameError:
-                    raise e
+                except NameError as ne:
+                    message = 'Failed to import module: {}'.format(ne.message)
 
-            # Print the error to the browser upon failure?
-            if settings.DEBUG:
-                # Return this unspecified exception as a 500.
-                content = "<!DOCTYPE html>500. From Zappa: <pre>" + str(e) + "</pre><br /><pre>" + traceback.format_exc().replace('\n', '<br />') + "</pre>"
-                exception = base64.b64encode(content)
-                raise Exception(exception)
-            else:
-                raise e
+            # Return this unspecified exception as a 500, using template that API Gateway expects.
+            content = collections.OrderedDict()
+            content['http_status'] = 500
+            body = {'message': message}
+            if settings.DEBUG:  # only include traceback if debug is on.
+                body['traceback'] = traceback.format_exception(*exc_info)  # traceback as a list for readability.
+            content['content'] = base64.b64encode(json.dumps(body, sort_keys=True, indent=4).encode('utf-8'))
+            exception = json.dumps(content)
+            raise UncaughtWSGIException(exception, original=e), None, exc_info[2]  # Keep original traceback.
 
-def lambda_handler(event, context): # pragma: no cover
+
+def lambda_handler(event, context):  # pragma: no cover
     return LambdaHandler.lambda_handler(event, context)
 
 
 def keep_warm_callback(event, context):
-    """This method is triggered by the CloudWatch event scheduled when keep_warm setting is set to true. """
+    """Method is triggered by the CloudWatch event scheduled when keep_warm setting is set to true."""
     lambda_handler(event={}, context=context)  # overriding event with an empty one so that web app initialization will
     # be triggered.
 
-def certify_callback(event, context):   
+
+def certify_callback(event, context):
     """
     Load our LH settings and update our cert.
     """
