@@ -17,6 +17,8 @@ import zipfile
 
 import kappa
 from distutils.dir_util import copy_tree
+
+from botocore.exceptions import ClientError
 from lambda_packages import lambda_packages
 from tqdm import tqdm
 
@@ -120,7 +122,14 @@ ATTACH_POLICY = """{
         {
             "Effect": "Allow",
             "Action": [
-                "ec2:CreateNetworkInterface"
+                "ec2:AttachNetworkInterface",
+                "ec2:CreateNetworkInterface",
+                "ec2:DeleteNetworkInterface",
+                "ec2:DescribeInstances",
+                "ec2:DescribeNetworkInterfaces",
+                "ec2:DetachNetworkInterface",
+                "ec2:ModifyNetworkInterfaceAttribute",
+                "ec2:ResetNetworkInterfaceAttribute"
             ],
             "Resource": "*"
         },
@@ -752,7 +761,7 @@ class Zappa(object):
                         content_mapping_templates[alias] = content_mapping_templates[content_type]
 
             if not self.credentials_arn:
-                self.get_credentials_arn()           
+                self.get_credentials_arn()
             credentials = self.credentials_arn  # This must be a Role ARN
 
             uri = 'arn:aws:apigateway:' + self.boto_session.region_name + ':lambda:path/2015-03-31/functions/' + lambda_arn + '/invocations'
@@ -1010,10 +1019,10 @@ class Zappa(object):
 
         return None
 
-    def create_domain_name( self, 
-                            domain_name, 
-                            certificate_name, 
-                            certificate_body, 
+    def create_domain_name( self,
+                            domain_name,
+                            certificate_name,
+                            certificate_body,
                             certificate_private_key,
                             certificate_chain,
                             lambda_name,
@@ -1075,9 +1084,9 @@ class Zappa(object):
         return response
 
     def update_domain_name( self,
-                            domain_name, 
-                            certificate_name, 
-                            certificate_body, 
+                            domain_name,
+                            certificate_name,
+                            certificate_body,
                             certificate_private_key,
                             certificate_chain,
                         ):
@@ -1087,7 +1096,7 @@ class Zappa(object):
 
         # Patch operations described here: https://tools.ietf.org/html/rfc6902#section-4
         # and here: http://boto3.readthedocs.io/en/latest/reference/services/apigateway.html#APIGateway.Client.update_domain_name
-        
+
         new_cert_name = 'LEZappa' + str(time.time())
         server_certificate = self.iam.create_server_certificate(
             ServerCertificateName=new_cert_name,
@@ -1272,9 +1281,9 @@ class Zappa(object):
                 svc = ','.join(event['event_source']['events'])
                 service = svc.split(':')[0]
 
-                self.create_event_permission(   
-                                                lambda_name, 
-                                                service + '.amazonaws.com', 
+                self.create_event_permission(
+                                                lambda_name,
+                                                service + '.amazonaws.com',
                                                 event['event_source']['arn']
                                             )
 
@@ -1348,6 +1357,8 @@ class Zappa(object):
         'events' is a list of dictionaries, where the dict must contains the string
         of a 'function' and the string of the event 'expression', and an optional 'name' and 'description'.
         """
+        self._clear_policy(lambda_name)
+
         rules = self.events_client.list_rules(NamePrefix=lambda_name)
         for rule in rules['Rules']:
             rule_name = rule['Name']
@@ -1370,6 +1381,32 @@ class Zappa(object):
                                             )
             print("Removed event " + name + ".")
 
+    def _clear_policy(self, lambda_name):
+        """
+        Removes obsolete policy statements to prevent policy from bloating over the limit after repeated updates.
+        """
+        try:
+            policy_response = self.lambda_client.get_policy(
+                FunctionName=lambda_name
+            )
+            if policy_response['ResponseMetadata']['HTTPStatusCode'] == 200:
+                statement = json.loads(policy_response['Policy'])['Statement']
+                for s in statement:
+                    delete_response = self.lambda_client.remove_permission(
+                        FunctionName=lambda_name,
+                        StatementId=s['Sid']
+                    )
+                    if delete_response['ResponseMetadata']['HTTPStatusCode'] != 204:
+                        logger.error('Failed to delete an obsolete policy statement: {}'.format())
+            else:
+                logger.debug('Failed to load Lambda function policy: {}'.format(policy_response))
+        except ClientError as e:
+            if e.message.find('ResourceNotFoundException') > -1:
+                logger.debug('No policy found, must be first run.')
+            else:
+                logger.error('Unexpected client error {}'.format(e.message))
+
+
     ##
     # CloudWatch Logging
     ##
